                  /*--.          .-"-. 
                 /   ._.        /  ` \
                 \_  (__\       \_v_/
                 //   \\        //   \\
                ((     ))      ((     ))
 --""---""----""---""--
                  |||            |||                                               
   Authors note:   |              |                                                
                                                                                   
   This file is part of work done by Emil Persson, aka Humus.                      
   You're free to use the code in any way you like, modified, unmodified           
   or cut'n'pasted into your own work. But for the good of the whole               
   programming community I will ask you to as long as possible and to the          
   extent you find suitable to follow these points:                                
    * If you use this file and it's contents unmodified, please leave this note.   
    * If you use a major part of the file in your app, please credit the author(s) 
      in one way or the other as you find suitable, for instance a small notice    
      in the readme file may do, or perhaps a link to the authors site.            
    * Share your work and ideas too as much as you can.                            
     _______                                                                       
   /` _____ `\;,    Humus                                                          
  (__(^===^)__)';,  emiper-8@student.luth.se                         ___           
    /  :::  \   ,;  ICQ #47010716                                  /^   ^\         
   |   :::   | ,;'                                                (     )        
 '._______.'`--oOo--(_)--oOo--*/

#ifndef _POLY_H_
#define _POLY_H_

#include "../Framework/Util/Light.h"

class Poly {
protected:
	Vertex v[4];
	TextureID bumpMapID;
	Vertex s,t,normal;
	float scaleS, scaleT;
public:
	Poly(){}

	Poly(Vertex v0, Vertex v1, Vertex v2, TextureID bumpMap, float scaleFactorS, float scaleFactorT){
		v[0] = v0;
		v[1] = v1;
		v[2] = v1 + v2 - v0;
		v[3] = v2;

		bumpMapID = bumpMap;

		scaleS = scaleFactorS;
		scaleT = scaleFactorT;

		s = v1 - v0;
		t = v2 - v0;
		s.normalize();
		t.normalize();
		normal = cross(s,t);
		
	}

	void drawLit(Renderer *renderer, TextureID vectorField, const Light &light, bool blend){
		renderer->setTextures(vectorField, bumpMapID);
		renderer->setTexEnvs(TEXENV_REPLACE, TEXENV_DOT3);
		if (blend) renderer->setBlending(GL_ONE_MINUS_DST_COLOR, GL_ONE);
		renderer->apply();
			
		glBegin(GL_QUADS);
		for (int i = 0; i < 4; i++){
			Vertex p = (light.getPosition() - v[i]) * 0.5f / light.getSize();
			glMultiTexCoord3fARB(GL_TEXTURE0_ARB, s * p + 0.5f, t * p + 0.5f, normal * p + 0.5f);
			glMultiTexCoord2fARB(GL_TEXTURE1_ARB, scaleS * (i == 1 || i == 2), scaleT * (i >= 2));
			
			glVertex3fv((GLfloat *) (v + i));
		}
		glEnd();
	}
};

#endif // _POLY_H_
